"""
@file   : color.py
@author : jiangmenggui@hosonsoft.com
@data   : 2024/4/2
"""
import re


class _Color:

    def __init__(self, s, start, end, name):
        self.raw = s
        self.s = start + s + end
        self.start = start
        self.end = end
        self.name = name

    def __len__(self):
        return self.raw.__len__()

    def __str__(self):
        return self.s.__str__()

    def __repr__(self):
        return self.s.__repr__()

    def __iter__(self):
        return self.raw.__iter__()

    def __add__(self, other):
        return self.s + other

    def __radd__(self, other):
        return other + self.s

    def __format__(self, format_spec):
        return self.start + self.raw.__format__(format_spec) + self.end


class _ColorFactory:

    def __init__(self, name, start, end='\033[0m'):
        self.name = name
        self.start = start
        self.end = end

    def __call__(self, s: str):
        return _Color(s, self.start, self.end, self.name)


class Color:
    UNDERLINE = '\033[4m'
    red = _ColorFactory('red', '\033[91m')
    green = _ColorFactory('green', '\033[92m')
    yellow = _ColorFactory('yellow', '\033[93m')
    blue = _ColorFactory('blue', '\033[94m')
    magenta = _ColorFactory('purple', '\033[95m')
    cyan = _ColorFactory('cyan', '\033[96m')
    white = _ColorFactory('white', '\033[97m')
    thin_red = _ColorFactory('red', '\033[31m')
    thin_green = _ColorFactory('green', '\033[32m')
    thin_yellow = _ColorFactory('yellow', '\033[33m')
    thin_blue = _ColorFactory('blue', '\033[34m')
    thin_magenta = _ColorFactory('purple', '\033[35m')
    thin_cyan = _ColorFactory('cyan', '\033[36m')
    thin_white = _ColorFactory('white', '\033[37m')

    success = green
    fail = red
    warning = yellow

    @classmethod
    def color(cls, val: str, style='i u r white on red'):
        colors = 'black red green yellow blue magenta cyan white'.split()
        style = re.split(r'\s+', style)
        values = set()
        for i in style:
            if i == 'i':
                values.add(3)
            elif i == 'u':
                values.add(4)
            elif i == 'r':
                values.add(7)
            elif i == 'on':
                next_color = style[style.index(i) + 1]
                values.add(colors.index(next_color) + 40)
            else:
                values.add(colors.index(i) + 30)

        return _ColorFactory('color', f'\033[{";".join(map(str, values))}m')(val)


if __name__ == '__main__':
    print(Color.red('hello world'))
    print(Color.green('hello world'))
    print(Color.yellow('hello world'))
    print(Color.blue('hello world'))
    print(Color.cyan('hello world'))
    print(Color.magenta('hello world'))
    print(Color.white('hello world'))
    print(Color.thin_red('hello world'))
    print(Color.thin_green('hello world'))
    print(Color.thin_yellow('hello world'))
    print(Color.thin_blue('hello world'))
    print(Color.thin_cyan('hello world'))
    print(Color.thin_magenta('hello world'))
    print(Color.thin_white('hello world'))
    print('ni hao ' + Color.red('world') + '!')
    print('|'.join(map(str, [Color.red('hello'), Color.green('world')])))
    print(Color.warning('warning: hello world'))
    print('\033[5;34;46mhello world\033[0m')
    print(Color.color('hello World', style='u black on yellow'))
    pass



# coding=utf-8
import logging
import os.path
from logging.handlers import TimedRotatingFileHandler

import colorlog

from my_tools.color import Color


class LogManager:

    def __init__(
            self,
            log_name,
            level=logging.DEBUG,  # 控制台中记录的日志等级
            formatter: str = None,  # 日志格式
            to_console=True,  # 是否打印到控制台
            console_colors=None,  # 控制台颜色，为None则使用默认值
            log_file: str = None,  # 日志文件记录路径，为None则默认记录在PYTHONPATH下的logs目录
            file_formatter: str = None,  # 日志文件中记录的格式
            file_level=None,  # 日志文件中的日志等级，为None则保持和level参数相同
            error_log_file: str = None,  # 错误日志文件记录路径
            error_level=logging.ERROR  # 错误日志记录的记录等级
    ):
        def _init_log_file_path(default_path):
            if 'PYTHONPATH' in os.environ:
                python_path = os.environ['PYTHONPATH']
                # 判断python_path是否为一个有效的路径
                if not os.path.exists(python_path):
                    print(Color.yellow(
                        f'可能处于DEBUG模式或python控制台运行，无法正确识别到当前工作目录，使用默认路径：{default_path}'))
                    return default_path
                log_path = os.path.join(python_path, 'logs')
                if not os.path.exists(log_path):
                    os.mkdir(path=log_path)
                return log_path
            print(Color.yellow(f'未读取到PYTHONPATH环境变量，使用默认路径：{default_path}'))
            return default_path

        if not formatter:
            formatter = '%(asctime)s.%(msecs)03d - %(name)s - "%(pathname)s:%(lineno)d" - ' \
                        '%(levelname)s - %(funcName)s : %(message)s'
        if not file_formatter:
            file_formatter = formatter
        self._colors = console_colors or {}
        self.formatter = '%(log_color)s' + formatter
        self._file_formatter = file_formatter
        if not file_level:
            file_level = level
        self._log_file = os.path.join(
            _init_log_file_path('/pythonlog'), f'out.log') if not log_file else log_file
        self._error_log_file = os.path.join(
            _init_log_file_path('/pythonlog'), f'error.log') if not error_log_file else error_log_file
        self._file_level = file_level
        self._error_file_level = error_level
        logging.root.setLevel(logging.NOTSET)
        self._logger = logging.getLogger(log_name)
        self._logger.setLevel(level)

        self._to_console = to_console
        self._console = to_console

    @staticmethod
    def _make_dir(path):
        parent_dir = os.path.dirname(os.path.abspath(path))
        if not os.path.exists(parent_dir):
            os.makedirs(parent_dir)

    def get_logger(self):
        if self._logger.handlers:
            return self._logger
        if self._to_console:
            colors = {
                'DEBUG': 'white',  # cyan white green black
                'INFO': 'blue',
                'WARNING': 'yellow',
                'ERROR': 'red',
                'CRITICAL': 'purple',
            }
            colors.update(self._colors)
            # 输出到控制台
            console_formatter = colorlog.ColoredFormatter(
                fmt=self.formatter,
                datefmt='%Y-%m-%d %H:%M:%S',
                log_colors=colors,
            )
            console_handler = logging.StreamHandler()
            console_handler.setFormatter(console_formatter)

            console_handler.setLevel(logging.DEBUG)

            self._logger.addHandler(console_handler)
            console_handler.close()

        if self._log_file:
            # 输出到文件
            file_formatter = logging.Formatter(
                fmt=self._file_formatter,
                datefmt='%Y-%m-%d  %H:%M:%S'
            )
            self._make_dir(self._log_file)
            file_handler = TimedRotatingFileHandler(
                filename=self._log_file,
                when='midnight',
                interval=1,
                backupCount=10,
                encoding='utf8',
                delay=True
            )
            # 日志级别，logger 和 handler以最高级别为准，不同handler之间可以不一样，不相互影响
            file_handler.setLevel(self._file_level)
            file_handler.setFormatter(file_formatter)
            self._logger.addHandler(file_handler)
            file_handler.close()
        if self._error_log_file:
            # 输出到文件
            file_formatter = logging.Formatter(
                fmt=self._file_formatter,
                datefmt='%Y-%m-%d %H:%M:%S'
            )
            self._make_dir(self._error_log_file)
            error_file_handler = TimedRotatingFileHandler(
                filename=self._error_log_file,
                when='midnight',
                interval=1,
                backupCount=10,
                encoding='utf8',
                delay=True
            )
            # 日志级别，logger 和 handler以最高级别为准，不同handler之间可以不一样，不相互影响
            error_file_handler.setLevel(self._error_file_level)
            error_file_handler.setFormatter(file_formatter)
            self._logger.addHandler(error_file_handler)
            error_file_handler.close()

        return self._logger


if __name__ == '__main__':
    logger2 = LogManager("my log").get_logger()

    logger2.debug("Hello World")
    logger2.info("Hello World")
    logger2.warning("Hello World")
    logger2.error("Hello World")
    logger2.critical("Hello World")



# coding=utf-8
import datetime
import threading
import time
from functools import wraps
from typing import Callable, Any, Type

from my_tools.color import Color
import traceback


def time_cache(__time: float, /):
    """
    给缓存数据设置超时时间，
    :param __time: 缓存时间
    :return:
    """
    __cache = {}

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            key = (func.__name__, *args, *tuple(kwargs.items()))
            if key in __cache and time.time() - __cache[key][0] <= __time:
                return __cache[key][1]
            res = func(*args, **kwargs)
            __cache[key] = (time.time(), res)
            return res

        return wrapper

    return decorator


def timer(
        __loop=1
):
    """
    函数运行计时，重复的运行函数多次，查看函数最终的运行时间
    :param __loop: 重复次数
    :return:
    """

    def outer(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            print(f'function [{Color.yellow(func.__name__)}] start running at {datetime.datetime.now()}')
            start = time.perf_counter()
            res = None
            for _ in range(__loop):
                res = func(*args, **kwargs)
            end = time.perf_counter()
            print(f'function [{Color.yellow(func.__name__)}]      finished at {datetime.datetime.now()}, '
                  f'loop {Color.green(str(__loop))} times, use time {Color.cyan(f'{(end - start) * 1000:.4f}')} ms')
            return res

        return wrapper

    return outer


def catch_exception(
        exceptions: list[Type[Exception]] | Type[Exception] = None,
        print_exception: Any | Callable = traceback.print_exception,
        retry_times: int = 0,
        retry_interval: float = 0,  # unit second
):
    """
    自动捕获异常
    :param exceptions: 异常类，默认为Exception
    :param print_exception: 输出异常的方法，默认traceback.print_exception
    :param retry_times: 重试次数，0表示不重试，默认值0
    :param retry_interval: 重试间隔，默认值0，单位秒
    :return:
    """
    if not exceptions:
        exceptions = [Exception]
    elif not isinstance(exceptions, list | tuple | set):
        exceptions = [exceptions]

    def outer(func):
        @wraps(func)
        def inner(*args, **kwargs):
            for i in range(retry_times + 1):
                try:
                    return func(*args, **kwargs)
                except tuple(exceptions) as e:

                    if i < retry_times:
                        print(
                            f'[{Color.yellow(func.__name__)}] {Color.magenta(e.__class__.__name__)}: {Color.red(str(e))}, {Color.green(f"retry {i + 1}/{retry_times}")}')
                        time.sleep(retry_interval)
                    else:
                        if callable(print_exception):
                            print_exception(e)

        return inner

    return outer


def retry(
        __retry: int = 1,
        /, *,
        interval: float = 0,  # unit second
        exceptions: list[Type[Exception]] | Type[Exception] = None,
):
    """
    自动重试（和catch_exception不同的在于最后一次没有成功不会捕获异常）
    :param __retry: 重试次数，0表示不重试，默认值1
    :param interval: 重试间隔，默认值0，单位秒
    :param exceptions: 异常类，默认为Exception，只有捕获到这个异常才会进行重试
    :return:
    """
    if not exceptions:
        exceptions = [Exception]
    elif not isinstance(exceptions, list | tuple | set):
        exceptions = [exceptions]

    def outer(func):
        @wraps(func)
        def inner(*args, **kwargs):
            for i in range(__retry):
                try:
                    return func(*args, **kwargs)
                except tuple(exceptions) as e:
                    print(f'[{Color.yellow(func.__name__)}] {Color.magenta(e.__class__.__name__)}: '
                          f'{Color.red(str(e))}, {Color.green(f"retry {i + 1}/{__retry}")}')

                    time.sleep(interval)
            return func(*args, **kwargs)

        return inner

    return outer


def debug(func):
    """
    执行函数时，自动打印函数执行信息
    :param func:
    :return:
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        args_str = (*args, *map(lambda x: f"{x[0]}={x[1]}", kwargs.items()))
        _f_info = (f'{Color.yellow(func.__name__)}{Color.blue('(')}'
                   f'{", ".join(map(str, map(Color.green, map(str, args_str))))}{Color.blue(')')}')
        now = datetime.datetime.now()
        t = time.perf_counter()

        res = func(*args, **kwargs)
        print(
            f'function [{Color.yellow(func.__name__)}] execute info: \n'
            f'  execute :  {_f_info}\n'
            f' start at :  {now}'
        )
        print(
            f'finish at :  {datetime.datetime.now()}\n'
            f'   result :  {str(res)[:500]}\n'
            f' use time :  {Color.cyan(f'{(time.perf_counter() - t) * 1000:.4f}')} ms'
        )
        return res

    return wrapper


def singleton(cls: str | type = ''):
    """
    实现单例模式
    :param cls:
    :return:
    """
    _instance = {}
    lock = threading.Lock()
    if isinstance(cls, str):
        def outer(cls_):
            @wraps(cls_)
            def inner(*args, **kwargs):
                # key = (cls_, getattr(cls_, cls, None))
                with lock:
                    key = kwargs.get(cls, (str(args[0]) if args else None))
                    key = (cls_, key)
                    if key not in _instance:
                        _instance[key] = cls_(*args, **kwargs)
                    return _instance[key]

            return inner

        return outer
    else:

        @wraps(cls)
        def wrapper(*args, **kwargs):
            with lock:
                if cls not in _instance:
                    _instance[cls] = cls(*args, **kwargs)
                return _instance[cls]

        return wrapper


def count_call(func):
    """
    统计函数被调用的次数
    :param func:
    :return:
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        wrapper.count += 1
        result = func(*args, **kwargs)
        print(f'[{Color.yellow(func.__name__)}]已被调用了 {Color.green(str(wrapper.count))} 次')
        return result

    wrapper.count = 0
    return wrapper


def rate_limited(__max_called_pre_second: float = 1, /, *, auto_wait=False):
    """
    限制函数被调用的频率
    :param __max_called_pre_second: 每秒最大调用次数
    :param auto_wait: 是否自动等待，True表示自动等待，False表示不等待，默认值为False
    :return:
    """
    if __max_called_pre_second <= 0:
        raise ValueError('__max_called_pre_second must > 0')

    interval = 1 / __max_called_pre_second

    def outer(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            now = time.time()
            if not wrapper.last_called:
                wrapper.last_called = now

            elif now - wrapper.last_called < interval:
                if auto_wait:
                    time.sleep(interval - (now - wrapper.last_called))
                else:
                    raise RuntimeError(f'[{Color.yellow(func.__name__)}]调用过于频繁，'
                                       f'每次调用间隔不得低于 {Color.green(f"{interval:.3f}")} 秒！')
            wrapper.last_called = time.time()
            return func(*args, **kwargs)

        wrapper.last_called = None
        return wrapper

    return outer


if __name__ == '__main__':
    from random import randint


    @timer(1000)
    @retry(2, interval=1)
    def f(a, b):
        return a / b


    @rate_limited(auto_wait=True)
    def random_value():
        val = randint(1, 3)
        if val != 3:
            raise ValueError('value must be 3')
        return val


    @debug
    @rate_limited(auto_wait=False)
    def add(a, b):
        return a + b


    add(1, 2)
    add(1, 2)



# coding=utf-8
import datetime
import sys

from my_tools.color import Color

print_raw = print


def track_print(*args, sep=' ', end='\n', file=None, flush=False):  # noqa
    fra = sys._getframe(1)  # noqa
    line = fra.f_lineno
    file_name = fra.f_code.co_filename
    t = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
    args = (
        f'[{Color.thin_green(t)}][{Color.thin_cyan("print")}]["{file_name}:{line}"]'
        f'[{Color.yellow(fra.f_code.co_name)}] :',
        *args)
    print_raw(sep.join(map(str, args)), file=file, flush=flush)


def patch_print():
    try:
        __builtins__.print = track_print
    except AttributeError:
        __builtins__['print'] = track_print


def restore_print():
    try:
        __builtins__.print = track_print
    except AttributeError:
        __builtins__['print'] = track_print


patch_print()
if __name__ == '__main__':
    def add():
        print('Hello World!')


    add()



# coding=utf-8

"""
@fileName       :   http_client.py
@data           :   2024/2/28
@author         :   jiangmenggui@hosonsoft.com
"""
from urllib.parse import urlparse
from http.client import HTTPConnection


class HTTPClient:

    def __init__(self, base_url: str = '', timeout=60):
        url = urlparse(base_url)
        self._timeout = timeout
        self._pool = {}


if __name__ == '__main__':
    pass



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="lljz_tools",
    version="0.2.11",
    author="liulangjuanzhou",
    author_email="liulangjuanzhou@gmail.com",
    description="常用工具封装",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    install_requires=['openpyxl'],
    classifiers=[
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# coding=utf-8

"""
@fileName       :   db_client.py
@data           :   2024/2/8
@author         :   jiangmenggui@hosonsoft.com
"""
import re

import pymysql
from dbutils.pooled_db import PooledDB
from pymysql.cursors import DictCursor

from my_tools.attribute_dict import AttributeDict
from my_tools.decorators import singleton


class DBConnection(AttributeDict, total=True, variable=False, check_type=True):
    host: str
    port: int
    database: str
    user: str = ''
    password: str = ''


class _MySQLContext:

    def __init__(self, con):
        self.con = con
        self.cur = self.con.cursor(DictCursor)
        self.cur.commit = self.con.commit
        self.cur.rollback = self.con.rollback

    def close(self):
        self.cur.close()
        self.con.close()

    def commit(self):
        self.con.commit()

    def rollback(self):
        self.con.rollback()

    def select_one(self, sql, params=None):
        self.cur.execute(sql, params)
        return self.cur.fetchone()

    def select_many(self, sql, params=None, size: int = None):
        count = self.cur.execute(sql, params)
        if size is None:
            if count <= 10_000:
                return self.cur.fetchall()
            raise ValueError('查询结果过多(>10w),，请指定many参数')
        return self.cur.fetchmany(size)

    def select_all(self, sql, params=None):
        return self.select_many(sql, params, None)

    def execute(self, sql, params=None):
        return self.cur.execute(sql, params)

    def executemany(self, sql, params):
        return self.cur.executemany(sql, params)

    def dml(self, sql: str, params=None, limit: int | None = 1000):
        t, *_ = sql.lstrip().split(maxsplit=1)
        if t.upper() not in ('INSERT', 'UPDATE', 'DELETE'):
            raise ValueError(f'DML操作不支持{t}语句')
        if params:
            params = tuple(params)
            if isinstance(params[0], tuple | set | list):
                count = self.executemany(sql, params)
            else:
                count = self.execute(sql, params)
        else:
            count = self.execute(sql)
        if limit and t.upper() != 'INSERT' and count > limit:
            self.rollback()
            raise ValueError(f'影响行数超过{limit}行')
        return count

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def start_transaction(self):
        return _TransactionManger(self.con)


@singleton('uri')
class DBClientPool:

    def __init__(self, uri: str, mincached=1, maxcached=5,
                 maxshared=5, maxconnections=100, blocking=False,
                 maxusage=None):
        """
        数据库连接
        :param uri: 连接串，格式如：mysql://user:password@host:port/database
        :param mincached: 连接池中空闲连接的初始数量
        :param maxcached: 连接池中空闲连接的最大数量
        :param maxshared: 共享连接的最大数量
        :param maxconnections: 创建连接池的最大数量
        :param blocking: 超过最大连接数量时候的表现，为True等待连接数量下降，为False直接报错处理
        :param maxusage: 单个连接的最大重复使用次数
        """
        obj = re.match(
            r'^mysql://(?P<user>.+):(?P<password>.+)@(?P<host>.+):(?P<port>\d+)/(?P<database>.+)$',
            uri
        )
        if not obj:
            raise ValueError('uri is wrong!')
        self.connection = DBConnection(obj.groupdict())
        self._pool = PooledDB(
            creator=pymysql,
            mincached=mincached,
            maxcached=maxcached,
            maxshared=maxshared,
            maxconnections=maxconnections,
            blocking=blocking,
            maxusage=maxusage,
            **self.connection
        )

    def connect(self) -> _MySQLContext:
        return _MySQLContext(self._pool.connection())

    def close(self):
        self._pool.close()


class _TransactionManger:

    def __init__(self, con):
        self.con = con

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.con.commit()


if __name__ == '__main__':
    # with MySQLClient('mysql://root:myroot123!@127.0.0.1:3306/server_monitor').connect() as client:
    #     client.execute('select * from information_schema.TABLES where TABLE_SCHEMA=%s', ['server_monitor'])
    #     print(client.fetchall())
    p1 = DBClientPool('mysql://root:Hosonsoft2020@192.168.1.220:3307/riin-platform')
    p2 = DBClientPool('mysql://root:Hosonsoft2020@192.168.1.220:3307/riin-platform')
    print(p1 is p2)
    with DBClientPool('mysql://root:Hosonsoft2020@192.168.1.220:3307/riin-platform').connect() as client:
        print(client.select_one("SELECT COUNT(*) FROM om_customer_order where id = %s", [1]))
        with client.start_transaction():
            client.dml("update om_customer_order set node_code = %s where id = %s", ['0701', 1])
